/*******************************************************************************
 * Copyright 2012 NetBurner, Inc.  ALL RIGHTS RESERVED
 *
 * Permission is hereby granted to purchasers of NetBurner hardware to use or
 * modify this computer program for any use as long as the resultant program is
 * only executed on NetBurner-provided hardware.
 *
 * No other rights to use this program or its derivatives in part or in whole
 * are granted.
 *
 * It may be possible to license this or other NetBurner software for use on
 * non-NetBurner hardware.  Please contact licensing@netburner.com for more
 * information.
 *
 * NetBurner makes no representation or warranties with respect to the
 * performance of this computer program, and specifically disclaims any
 * responsibility for any damages, special or consequential, connected with use
 * of this program.
 *
 * ----------------------------------------------------------------------------
 *
 * NetBurner, Inc.
 * 5405 Morehouse Drive
 * San Diego, CA  92121
 *
 * Information available at:  http://www.netburner.com
 * Support available at:      http://support.netburner.com
 *
 ******************************************************************************/
/*-----------------------------------------------------------------------------
 * MOD54415 DAC Examples
 *
 * This example illustrates how to use the DAC0 signal on pin J2.9.
 * There are two modes demonstrated:
 * 1. Step up the output from 0 to 0xFFF by writing to the DAC data register.
 * 2. Use the auto waveform generator feature synced to DMA timer 0 to step
 *    up the output.
 *
 * IMPORTANT: You must have the analog reference voltage pin connected to
 * something between 0 and 3.3VDC. The easiest way is to connect 3.3VDC by
 * jumpering J2.2 (Vcc) to J2.5 (Analog ref. input).
 *
 *------------------------------------------------------------------------------*/

#include "predef.h"
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <smarttrap.h>
#include <NetworkDebug.h>
#include <sim.h>
#include <basictypes.h>
#include <pins.h>


extern "C" {
   void UserMain(void * pd);
}

const char * AppName = "MOD54415 DAC";

// Sim1 Miscellaneous Control Register
#define MISCCR2_DAC1SEL 0x0040   // Enable DAC1 drive output
#define MISCCR2_DAC0SEL 0x0020   // Enable DAC0 drive output
#define MISCCR2_ADCEN   0x0010   // Enable ADC 6-4 and 2-0
#define MISCCR2_ADC7EN  0x0008   // Enable ADC 7
#define MISCCR2_ADC3EN  0x0004   // Enable ADC 3

// ADC Calibration Register
#define ADC_CAL_DAC1  0x0002     // Selects the source of the ADCA7 input as DAC1 output.
#define ADC_CAL_DAC0  0x0001     // Selects the source of the ADCA3 input as DAC0 output.

// DAC Control Register
#define DAC_CR_RESET 0x1101
#define DAC_CR_UP    0x0020
#define DAC_CR_DOWN  0x0010
#define DAC_CR_AUTO  0x0008
#define DAC_CR_SYNC  0x0004
#define DAC_CR_FMT   0x0002
#define DAC_CR_PDN   0x0001       // Power down. 0 = power on, 1 = power down

// DAC Trigger Select Register DACTSR
// Channels
#define DAC1_CH0  0x0000               // Source channel 0
#define DAC1_CH1  0x0800               // Source channel 1
#define DAC1_CH2  0x1000               // Source channel 2
#define DAC1_CH3  0x1800               // Source channel 3

#define DAC0_CH0  0x0000               // Source channel 0
#define DAC0_CH1  0x0008               // Source channel 1
#define DAC0_CH2  0x0010               // Source channel 2
#define DAC0_CH3  0x0018               // Source channel 3

// Sources
#define DAC1_SRC_PWMA      0x0000      // PWMs
#define DAC1_SRC_PWMB      0x0100
#define DAC1_SRC_PWMX      0x0200
#define DAC1_SRC_PWMTRIG1  0x0300
#define DAC1_SRC_PWMTRIG0  0x0400
#define DAC1_SRC_TnOUT     0x0500      // Timers
#define DAC1_SRC_TnIN      0x0600

#define DAC0_SRC_PWMA      0x0000      // PWMs
#define DAC0_SRC_PWMB      0x0001
#define DAC0_SRC_PWMX      0x0002
#define DAC0_SRC_PWMTRIG1  0x0003
#define DAC0_SRC_PWMTRIG0  0x0004
#define DAC0_SRC_TnOUT     0x0005      // Timers
#define DAC0_SRC_TnIN      0x0006



/*-----------------------------------------------------------------------------
 * Simple DAC example illustrating how to enable DAC0 and write a sequence
 * of output values until a key is pressed to exit.
 *-----------------------------------------------------------------------------*/
void SimpleDacOut()
{
   sim1.ccm.dactsr = 0x0;        // DAC trigger select register in CCM
   sim2.adc.cal = ADC_CAL_DAC0;  // Selects the source of the ADCA3 input as DAC0 output.
   sim2.dac[0].data = 0;         // Init data value to 0

   sim1.ccm.misccr2 &= ~(MISCCR2_ADC3EN);   // Disable ADC3 function on J2.9
   sim1.ccm.misccr2 |= MISCCR2_DAC0SEL;     // Enable DAC0 output drive on J2.9

   sim2.dac[0].cr &= ~DAC_CR_PDN;           // Enable DAC0 output

   iprintf("DAC0 ramp starting, hit any key to exit\r\n");

   char c = 0;
   while( c == 0 )
   {
      sim2.dac[0].data += 0x0010;
      if ( sim2.dac[0].data >= 0x0FFF ) sim2.dac[0].data = 0;

      //iprintf("Data: 0x%04X\r", sim2.dac[0].data );

      if ( charavail() )
         c = getchar();
   }

   iprintf("\r\n");
}



void StopDmaTimer( int timer )
{
   sim2.timer[timer].tmr = 0;  // stop counting
   sim2.timer[timer].tcn = 0x0;      // reset free run counter
   sim2.timer[timer].trr = 0x0;      // reset free run counter
   sim2.timer[timer].ter = 0x3;      // clear timer event registers
}



/*------------------------------------------------------------------
 * Start a DMA timer. Used for DAC sync_in inupt
 *-----------------------------------------------------------------*/
extern unsigned long CPU_CLOCK;

int StartDmaTimer( float frequency, int timer )
{
   if ( (frequency <= 0 ) || ( timer < 0 ) || (timer > 3 ) )
   {
      iprintf("Error: frequency: %f, timer: %d\r\n", frequency, timer );
      return -1;
   }

   sim2.timer[timer].tmr = 0;  // Reset timer

   float ToggleTime = (1.0 / frequency ) * 0.5;
   float CpuClockPeriod = ( 1.0 / (CPU_CLOCK/2) );
   unsigned int RefTimerCount = (unsigned int) ( ToggleTime / CpuClockPeriod );

   //iprintf("Number of timer ref counts: %d\r\n", RefTimerCount );

   sim2.timer[timer].tmr = 0x002A;  // Toggle output, reset count after reaching reference
   sim2.timer[timer].txmr = 0x00;
   sim2.timer[timer].trr = RefTimerCount;
   sim2.timer[timer].tmr |= 0x0001;  // start timer

   return 0;
}


/*-----------------------------------------------------------------------------
 * Example of how to create a waveform using DMA timer 0 as a trigger.
 *
 *-----------------------------------------------------------------------------*/
void DacWaveformWithTimerTrigger()
{
   sim1.ccm.dactsr = 0x0;        // DAC trigger select register in CCM
   sim2.adc.cal = ADC_CAL_DAC0;  // Selects the source of the ADCA3 input as DAC0 output.
   sim2.dac[0].data = 0;         // Init data value to 0

   sim1.ccm.misccr2 &= ~(MISCCR2_ADC3EN);   // Disable ADC3 function on J2.9
   sim1.ccm.misccr2 |= MISCCR2_DAC0SEL;     // Enable DAC0 output drive on J2.9

   sim1.ccm.dactsr |= (DAC0_CH0 | DAC0_SRC_TnOUT);

   // Used for advanced function of auto-generation of a waveform
   sim2.dac[0].step = 0x0100;
   sim2.dac[0].max  = 0x0FFF;
   sim2.dac[0].min  = 0x0000;

   sim2.dac[0].cr |= DAC_CR_UP | DAC_CR_AUTO | DAC_CR_SYNC;

   sim2.dac[0].cr &= ~DAC_CR_PDN;           // Enable DAC0 output

   iprintf("Waveform on DAC0 (J2.36) starting, hit any key to exit\r\n");

   char c = 0;
   while( c == 0 )
   {
      OSTimeDly( TICKS_PER_SECOND / 4 );

      if ( charavail() )
         c = getchar();
   }
}

/*-----------------------------------------------------------------------------
 * Display Menu
 *----------------------------------------------------------------------------*/
void DisplayMenu()
{
   iprintf("\r\n\r\n");
   iprintf("1. Simple DAC output writing to DAC data register\r\n");
   iprintf("2. DAC waveform generation using DMA Timer 0\r\n");
}



/*-----------------------------------------------------------------------------
 * UserMain
 *----------------------------------------------------------------------------*/
void UserMain(void * pd)
{
    InitializeStack();
    if (EthernetIP == 0) GetDHCPAddress();
    OSChangePrio(MAIN_PRIO);
    EnableAutoUpdate();

    #ifndef _DEBUG
    EnableSmartTraps();
    #endif

    #ifdef _DEBUG
    InitializeNetworkGDB_and_Wait();
    #endif

    iprintf("Application started\n");
    iprintf("IMPORTANT: The DAC will not work unless the analog reference J2.5 is connected!\r\n");

    while(1)
    {
       DisplayMenu();
       char c = getchar();
       switch ( c )
       {
       case '1':
          SimpleDacOut();
          break;

       case '2':
          {
          float TimerFreq = 100000;
          int TimerChannel = 0;
          iprintf("Enabling DMA Timer 0 output on J2.36. Freq = %d\r\n", TimerFreq);
          J2[36].function( PINJ2_36_T0OUT );
          StartDmaTimer( TimerFreq, TimerChannel );
          DacWaveformWithTimerTrigger();
          StopDmaTimer( TimerChannel );
          }
          break;

       default:
                DisplayMenu();
       }

       sim2.dac[0].cr = DAC_CR_PDN;           // Disable DAC0 output
    }
}





